/*
								+----------------------------------+
								|                                  |
								|  ***   Simple FPS counter   ***  |
								|                                  |
								|   Copyright  -tHE SWINe- 2006   |
								|                                  |
								|          FPS_Counter.h           |
								|                                  |
								+----------------------------------+
*/

#ifndef __FPS_COUNTER_INCLUDED
#define __FPS_COUNTER_INCLUDED

class CFPSCounter {
protected:
	int m_n_frame_num;
	float m_f_last_fps_time;
	float m_f_last_frame_time;
	float m_f_fps_avg;
	float m_f_fps_savg;
	float m_f_fps_cur;
	float m_f_avg_fps_poll_time;
	float m_f_avg_slide_coeff;

public:
	/*
	 *	CFPSCounter::CFPSCounter(float f_avg_fps_poll_time = 1.0f, float f_avg_slide_coeff = .05f)
	 *		- default constructor
	 *		- f_avg_fps_poll_time is period (in seconds) by which average FPS will be upgraded
	 *		  (the bigger period, the more precise reading)
	 *		- f_avg_slide_coeff is coefficient for calculating sliding-average FPS
	 *		  (it's calculated from current-frame FPS by averaging in time; the smaller number,
	 *		  the slower propagation of FPS changes, but more smoother average)
	 */
	CFPSCounter(float f_avg_fps_poll_time = 1.0f, float f_avg_slide_coeff = .1f);

	/*
	 *	void CFPSCounter::Reset()
	 *		- resets internal time counters
	 */
	void Reset();

	/*
	 *	void CFPSCounter::OnFrame(float f_frame_time)
	 *		- this function needs to be called every frame
	 *		- f_frame_time is continous frame-time
	 *		- in case f_frame_time is less than previous frame time
	 *		  (might be caused by timer reseting, etc.) FPS counter resets itself
	 */
	void OnFrame(float f_frame_time);

	/*
	 *	__inline float CFPSCounter::f_AVG_FPS() const
	 *		- returns average number of frames per second (average on a given time lapse)
	 */
	__inline float f_AVG_FPS() const
	{
		return m_f_fps_avg;
	}

	/*
	 *	__inline float CFPSCounter::f_Sliding_AVG_FPS() const
	 *		- resturns sliding average nomber of frames per second
	 */
	__inline float f_Sliding_AVG_FPS() const
	{
		return m_f_fps_savg;
	}

	/*
	 *	__inline float CFPSCounter::f_Cur_FPS() const
	 *		- returns current (extrapolated) number of frames per second
	 *		- note this reading is rather noisy
	 */
	__inline float f_Cur_FPS() const
	{
		return m_f_fps_cur;
	}
};

class CTimeLineGraph {
protected:
	struct TTimeLineInfo {
	protected:
		float *p_queue;
		int n_queue_length;

	public:
		int n_id;
		float f_min_value;
		float f_max_value;
		bool b_normalize;
		bool b_solid;
		float f_opacity;
		Vector3f v_color;

		TTimeLineInfo();
		~TTimeLineInfo();
		bool Alloc(int n_size);
		int n_Length() const;
		bool AddReading(float f_value, int n_skip = 0);
		void GL_Draw(CGLState *p_state, Vector2f v_min, Vector2f v_max,
			float f_min, float f_max, float f_subframe_time) const;
		void GL_DrawFill(CGLState *p_state, Vector2f v_min, Vector2f v_max,
			float f_min, float f_max, float f_subframe_time) const;
		float f_Cur_MinSpike() const;
		float f_Cur_MaxSpike() const;
	};

	std::vector<TTimeLineInfo*> m_timeline_list; // sorted in z-order
	int m_n_queue_id_pool;

	class CIfIdEquals {
	protected:
		int m_n_id;

	public:
		__inline CIfIdEquals(int n_id);
		__inline bool operator ()(const TTimeLineInfo *p_info) const;
	};

	class CDeleteTimeline {
	public:
		__inline void operator ()(const TTimeLineInfo *p_info) const;
		__inline operator bool() const;
	};

	class CRenderTimeLine {
	protected:
		bool m_b_allow_separate_scale;
		bool m_b_allow_separate_zero;
		bool m_b_first;
		float m_f_min, m_f_max, m_f_size;
		float m_f_subframe_time;
		Vector2f m_v_min, m_v_max;
		CGLState *m_p_state;

	public:
		__inline CRenderTimeLine(CGLState *p_state, bool b_allow_separate_scale,
			bool b_allow_separate_zero, Vector2f v_min, Vector2f v_max, float f_subframe_time);
		__inline CRenderTimeLine(const CRenderTimeLine &r_render);
		__inline void operator ()(const TTimeLineInfo *p_info);
		__inline operator bool() const;
	};

	class CRenderTimeLineRender : public CRenderTimeLine {
	public:
		__inline CRenderTimeLineRender(const CRenderTimeLine &r_render);
		__inline void operator ()(const TTimeLineInfo *p_info);
	};

public:
	/*
	 *	CTimeLineGraph::CTimeLineGraph()
	 *		- default constructor
	 */
	CTimeLineGraph();

	/*
	 *	CTimeLineGraph::~CTimeLineGraph()
	 *		- default destructor
	 */
	~CTimeLineGraph();

	/*
	 *	int CTimeLineGraph::n_AddTimeline(int n_buffer_length, float f_min_value,
	 *		float f_max_value, bool b_normalize, Vector3f v_color,
	 *		bool b_solid, float f_opacity)
	 *		- add a new timeline
	 *		- n_buffer_length is length of buffer (min 2)
	 *		- f_min_value and f_max_value are min/max values,
	 *		  timeline graph will be stretched between those two,
	 *		  values out of range will be clamped
	 *		- if b_normlize is true, actual min and max value
	 *		  to be used for drawing will be determined from
	 *		  minimal and maximal value in timeline window
	 *		- v_color is color to draw timeline with
	 *		- b_solid indicates wheter timeline should be drawn with simple line (0)
	 *		  or as area, enclosed between time axis and graph line
	 *		- f_opacity indicates opacity for rendering solid timeline
	 *		  (solid timeline is drawn as solid line and optionaly with
	 *		  transparent fill)
	 *		- returns timeline id in case of success or -1 on failure
	 */
	int n_AddTimeline(int n_buffer_length, float f_min_value,
		float f_max_value, bool b_normalize, Vector3f v_color,
		bool b_solid = false, float f_opacity = 1);

	/*
	 *	bool CTimeLineGraph::b_IsTimeline(int n_id)
	 *		- returns true if n_id specifies valid timeline. otherwise false
	 */
	bool b_IsTimeline(int n_id);

	/*
	 *	bool CTimeLineGraph::DeleteTimeline(int n_id)
	 *		- deletes timeline specified by n_id
	 *		- returns true if n_id specified valid timeline. otherwise false
	 */
	bool DeleteTimeline(int n_id);

	/*
	 *	void CTimeLineGraph::DeleteTimelines()
	 *		- delete all timelines
	 */
	void DeleteTimelines();

	/*
	 *	bool CTimeLineGraph::AddReading(int n_id, float f_value, int n_skip = 0)
	 *		- adds reading to timeline specified by n_id
	 *		- f_value is reading value
	 *		- n_skip is number of time frames to skip (all will be set to f_value)
	 */
	bool AddReading(int n_id, float f_value, int n_skip = 0);

	/*
	 *	void CTimeLineGraph::RenderTimelineWindow(GLState *p_state, Vector2f v_min,
	 *		Vector2f v_max, bool b_allow_separate_scale, bool b_allow_separate_zero,
	 *		Vector3f v_border_color, Vector3f v_back_color, float f_back_opacity,
	 *		float f_subframe_time = 0)
	 *		- render rectangular timeline window using OpenGL
	 *		- p_state is OpenGL state manager
	 *		- v_min and v_max are coordinates of rectangluar window
	 *		- if b_allow_separate_scale is true, different timelines can be rendered with different
	 *		  (maximal) scale so the values in the graph might not correspond to each other
	 *		- if b_allow_separate_zero is true, different timelines can have time axis at different
	 *		  vertical position
	 *		- v_border_color is color for graph border and for time axes
	 *		- v_back_color and f_back_opacity affect rendering of background
	 *		- f_subframe_time is relative time until next time frame in range <0, 1> allowing
	 *		  graph to animate smoothly
	 */
	void RenderTimelineWindow(CGLState *p_state, Vector2f v_min,
		Vector2f v_max, bool b_allow_separate_scale, bool b_allow_separate_zero,
		Vector3f v_border_color, Vector3f v_back_color, float f_back_opacity,
		float f_subframe_time = 0);

	/*
	 *	float CTimeLineGraph::f_Min_Value(int n_id)
	 *		- returns minimal value from current values in timeline window for
	 *		  timeline specified by n_id
	 *		- returns 0 in cas n_id doesn't specify a valid timeline
	 *		- note this is actual minimal reading value, not minimal value specified in
	 *		  n_AddTimeline() or by SetTimeline_Min()
	 */
	float f_Min_Value(int n_id);

	/*
	 *	float CTimeLineGraph::f_Max_Value(int n_id)
	 *		- returns maximal value from current values in timeline window for
	 *		  timeline specified by n_id
	 *		- returns 0 in cas n_id doesn't specify a valid timeline
	 *		- note this is actual maximal reading value, not maximal value specified in
	 *		  n_AddTimeline() or by SetTimeline_Max()
	 */
	float f_Max_Value(int n_id);

	/*
	 *	int CTimeLineGraph::n_Timeline_Num() const
	 *		- returns number of timelines being currently displayed
	 */
	int n_Timeline_Num() const;

	/*
	 *	int CTimeLineGraph::n_GetTimeline_ZOrder(int n_id) const
	 *		- returns z-order of timeline specified by n_id (it's order timelines are drawn in;
	 *		  it's number in range <0, n_Timeline_Num() - 1>)
	 *		- returns -1 in case n_id doesn't specify a valid timeline
	 */
	int n_GetTimeline_ZOrder(int n_id) const;

	/*
	 *	int CTimeLineGraph::n_GetTimeline_Length(int n_id) const
	 *		- returns queue length of timeline specified by n_id
	 *		- returns -1 in case n_id doesn't specify a valid timeline
	 */
	int n_GetTimeline_Length(int n_id) const;

	/*
	 *	bool CTimeLineGraph::b_GetTimeline_Normalize(int n_id) const
	 *		- returns the in case timeline specified by n_id is to be drawn normalized
	 *		- returns false in case n_id doesn't specify a valid timeline
	 */
	bool b_GetTimeline_Normalize(int n_id) const;

	/*
	 *	bool CTimeLineGraph::b_GetTimeline_Normalize(int n_id) const
	 *		- returns true in case timeline specified by n_id is to be drawn solid
	 *		- returns false in case n_id doesn't specify a valid timeline
	 */
	bool b_GetTimeline_Solid(int n_id) const;

	/*
	 *	float CTimeLineGraph::f_GetTimeline_Min(int n_id) const
	 *		- returns minimal graph value of timeline specified by n_id
	 *		  (note it's not an actual value in timeline window; just
	 *		  minimal value used to scale the graph in case normlization
	 *		  for a given timeline is not enabled)
	 *		- returns 0 in case n_id doesn't specify a valid timeline
	 */
	float f_GetTimeline_Min(int n_id) const;

	/*
	 *	float CTimeLineGraph::f_GetTimeline_Max(int n_id) const
	 *		- returns maximal graph value of timeline specified by n_id
	 *		  (note it's not an actual value in timeline window; just
	 *		  maximal value used to scale the graph in case normlization
	 *		  for a given timeline is not enabled)
	 *		- returns 0 in case n_id doesn't specify a valid timeline
	 */
	float f_GetTimeline_Max(int n_id) const;

	/*
	 *	Vector3f CTimeLineGraph::v_GetTimeline_Color(int n_id) const
	 *		- returns color of timeline specified by n_id
	 *		- returns vector of zeros (black) in case n_id doesn't specify a valid timeline
	 */
	Vector3f v_GetTimeline_Color(int n_id) const;

	/*
	 *	float CTimeLineGraph::f_GetTimeline_Opacity(int n_id) const
	 *		- returns opacity of timeline specified by n_id
	 *		- returns 0 in case n_id doesn't specify a valid timeline
	 */
	float f_GetTimeline_Opacity(int n_id) const;

	/*
	 *	bool CTimeLineGraph::SetTimeline_ZOrder(int n_id, int n_z_order)
	 *		- set z-order of timeline, specified by n_id
	 *		- returns true if n_id specified valid timeline. otherwise false
	 */
	bool SetTimeline_ZOrder(int n_id, int n_z_order);

	/*
	 *	bool CTimeLineGraph::SetTimeline_Length(int n_id, int n_length)
	 *		- set length of timeline, specified by n_id
	 *		- returns true if n_id specified valid timeline and there
	 *		  was enough memory for new queue. otherwise false
	 */
	bool SetTimeline_Length(int n_id, int n_length);

	/*
	 *	bool CTimeLineGraph::SetTimeline_Normalize(int n_id, bool b_normalize)
	 *		- set normalization of timeline, specified by n_id
	 *		- returns true if n_id specified valid timeline. otherwise false
	 */
	bool SetTimeline_Normalize(int n_id, bool b_normalize);

	/*
	 *	bool CTimeLineGraph::SetTimeline_Solid(int n_id, bool b_solid)
	 *		- set fill style of timeline, specified by n_id to b_solid
	 *		- returns true if n_id specified valid timeline. otherwise false
	 */
	bool SetTimeline_Solid(int n_id, bool b_solid);

	/*
	 *	bool CTimeLineGraph::SetTimeline_Min(int n_id, float f_min)
	 *		- set minimal value of graph y-axis of timeline, specified by n_id
	 *		- returns true if n_id specified valid timeline. otherwise false
	 */
	bool SetTimeline_Min(int n_id, float f_min);

	/*
	 *	bool CTimeLineGraph::SetTimeline_Max(int n_id, float f_max)
	 *		- set maximal value of graph y-axis of timeline, specified by n_id
	 *		- returns true if n_id specified valid timeline. otherwise false
	 */
	bool SetTimeline_Max(int n_id, float f_max);

	/*
	 *	bool CTimeLineGraph::SetTimeline_Color(int n_id, Vector3f v_color)
	 *		- set color of timeline, specified by n_id
	 *		- returns true if n_id specified valid timeline. otherwise false
	 */
	bool SetTimeline_Color(int n_id, Vector3f v_color);

	/*
	 *	bool CTimeLineGraph::SetTimeline_Opacity(int n_id, float f_opacity)
	 *		- set opacity of fill of timeline, specified by n_id
	 *		- returns true if n_id specified valid timeline. otherwise false
	 */
	bool SetTimeline_Opacity(int n_id, float f_opacity);

protected:
	__inline std::vector<TTimeLineInfo*>::iterator p_TimeLine(int n_id);
	__inline std::vector<TTimeLineInfo*>::const_iterator p_TimeLine(int n_id) const;
};

#endif //__FPS_COUNTER_INCLUDED
